//+------------------------------------------------------------------+
//|                                                    Functions.mqh |
//|                                    2019-2021, dimitri pecheritsa |
//|                                         mql5.com/en/users/dmipec |
//|------------------------------------------------------------------|
//| fnc | miscellaneous                                              |
//|------------------------------------------------------------------|
//| use | selection sort example                                     |
//|------------------------------------------------------------------|
//| 010 | deals load (deals[], from, to)                             |
//| 020 | deals key type (deals[o], keys[i])                         |
//| 030 | deals print (deals[])                                      |
//| 040 | array sort (keys[], items[], algorithm, order)             |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| 010 | deals load                                                 |
//|------------------------------------------------------------------|
//| out | tickets  | destination array                               |
//| in  | from, to | range of dates. by default loads all deals      |
//|available in the account                                          |
//| ret | number of copied deals in the specified date range. -1 is  |
//|the history select error                                          |
//+------------------------------------------------------------------+
int DealsLoad(ulong& dst_tickets[],datetime from=0,datetime to=0)
  {
   to=(to==0)? TimeCurrent() : to;
   if(!HistorySelect(from,to))
      return -1;
   int total=HistoryDealsTotal(),count=0;
   for(int i=0; i<total; i++)
     {
      ulong iticket=HistoryDealGetTicket(i);
      if(!iticket)
         continue;
      int size=ArraySize(dst_tickets);
      ArrayResize(dst_tickets,size+1,size*2);
      dst_tickets[size]=iticket;
      count++;
     }
   return count;
  }
//+------------------------------------------------------------------+
//| 020 | deals key type                                             |
//|------------------------------------------------------------------|
//| in  | deals | source array with deal tickets                     |
//| out | keys  | destination array for deal type values             |
//| ret | number of items copied. the destination 'keys' is filled   |
//|with the deals type values which are extracted from the delas     |
//+------------------------------------------------------------------+
void DealsKeyType(ulong& deals[],int& keys[])
  {
   int total=ArraySize(deals);
   ArrayResize(keys,total);
   for(int i=0; i<total; i++)
      keys[i]=(!HistoryDealSelect(deals[i]))?
              (int)WRONG_VALUE :
              (int)HistoryDealGetInteger(deals[i],DEAL_TYPE);
  }
//+------------------------------------------------------------------+
//| 030 | deals print                                                |
//|------------------------------------------------------------------|
//| in  | deals | source array with the deal tickets                 |
//| use | print a table of deals consiting of three columns: deal    |
//|ticket, symbol, deal type                                         |
//+------------------------------------------------------------------+
void DealsPrint(ulong& deals[])
  {
//--- format table columns
   string format="%21s | %13s | %13s";
//--- create horizontal table line
   string line;
   StringInit(line,68,'-');
//--- header
   printf(line);
   printf(format,"deal","symbol","type");
   printf(line);
//--- body
   int total=ArraySize(deals);
   for(int i=0; i<total; i++)
     {
      //--- some data for the deal
      bool select=HistoryDealSelect(deals[i]);
      string symbol=(!select)? "" : HistoryDealGetString(deals[i],DEAL_SYMBOL);
      ENUM_DEAL_TYPE type=(!select)?
                            (ENUM_DEAL_TYPE)WRONG_VALUE :
                            (ENUM_DEAL_TYPE)HistoryDealGetInteger(deals[i],DEAL_TYPE);
      //--- out
      printf(format,(string)deals[i],symbol,EnumToString(type));
     }
//--- footer
   printf(line);
  }
//+------------------------------------------------------------------+
//| 040 | array sort                                                 |
//|------------------------------------------------------------------|
//| i/o | keys      | array which values are compared and sorted     |
//| i/o | items     | array which items are swapped, and thus sorted |
//|the same way as the keys array. 'items' are ignored if its length |
//|does not match the length of the 'keys'                           |
//| in  | algorithm | any compatible sorting algorithm               |
//| in  | order     | sorting order flag, accending true / descending|
//|false                                                             |
//| use | sorts 'items' by 'keys' with the specified concrete sorting|
//|algorithm, in accending or descending order                       |
//+------------------------------------------------------------------+
#include <Mqh\Algorithms\ASorter.mqh>
template<typename TKey,typename TItem>
void ArraySort(TKey& keys[],TItem& items[],ASorter<TKey,TItem>* algorithm,bool accending_order=true)
  {
//--- check
   if(CheckPointer(algorithm)==POINTER_INVALID)
      return;
//--- prepare algorithm
   ArrayCopy(algorithm.keys,keys);
   ArrayCopy(algorithm.items,items);
//--- execute algorithm sorting
   algorithm.Sort();
//--- save sorted items
   ArrayCopy(keys,algorithm.keys);
   ArrayCopy(items,algorithm.items);
//--- keep sorting order
   if(!accending_order)
     {
      ArrayReverse(keys);
      ArrayReverse(items);
     }
//--- clear
   if(CheckPointer(algorithm)==POINTER_DYNAMIC)
      delete algorithm;
  }
//+------------------------------------------------------------------+
